/*
** ifstat.c - Display stats of network interfaces
**
** Author: Peter Eriksson (pen@lysator.liu.se) 11 Feb 1991
*/

#include <stdio.h>
#include <errno.h>
#include <ctype.h>
#include <netdb.h>
#include <nlist.h>
#include <paths.h>

#include <sys/types.h>
#include <sys/socket.h>
#include <sys/ioctl.h>

#include <net/if.h>
#include <netinet/in.h>

#define VERSION "[ifstat, version 1.00 - (c) 1991 Lysator]"

#define SINADDR(x) (((struct sockaddr_in *) &(x))->sin_addr)




struct nlist nl[] =
{
#define N_IFNET 0  
  { "_ifnet" },
  { "" }
};

FILE *fp;
int verbose_flag = 0;



getbuf(addr, buf, len)
  long addr;
  char *buf;
  int len;
{
  if (fseek(fp, addr, 0) < 0)
  {
    perror("ifstat: fseek(getbuf)");
    exit(1);
  }

  if (fread(buf, len, 1, fp) != 1)
  {
    perror("ifstat: fread(getbuf)");
    exit(1);
  }
}




showinfo(ifn)
  struct ifnet *ifn;
{
  char name[30];
  char buf[80];
  char *cp;
  struct netent *nep;
  int pos = 0;
  

  getbuf((long) ifn->if_name, name, sizeof(name));
  sprintf(buf, "%s%d", name, ifn->if_unit);
  printf("%s", buf);
  pos = strlen(buf);
  
  if (!(ifn->if_flags & IFF_RUNNING))
  {
    pos++;
    putchar('*');
  }

  while (pos < 6)
  {
    pos++;
    putchar(' ');
  }

  printf("%-5d ", ifn->if_mtu);

  if (ifn->if_flags & IFF_POINTOPOINT)
  {
    printf("%-15s ", inet_ntoa(SINADDR(ifn->if_dstaddr)));
    if (verbose_flag)
      printf("%-15s ", "");
  }
  else
  {
    sprintf(buf, "%s", inet_ntoa(htonl(ifn->if_net)));
    for (cp = buf; cp[0] == '0' && cp[1] == '.'; cp += 2)
      ;
    printf("%-15s ", cp);

    if (verbose_flag)
      if (ifn->if_flags & IFF_BROADCAST)
	printf("%-15s ", inet_ntoa(SINADDR(ifn->if_broadaddr)));
      else
	printf("%-15s ", "");
  }

  printf("%-15s ", inet_ntoa(SINADDR(ifn->if_addr)));

  printf("%-7d %-5d %-7d %-5d %-6d",
	 ifn->if_ipackets,
	 ifn->if_ierrors,
	 ifn->if_opackets,
	 ifn->if_oerrors,
	 ifn->if_collisions);

  if (verbose_flag)
  {
    printf(" %-4d", ifn->if_timer);
    printf(" %-4d", ifn->if_snd.ifq_len);
    printf(" %-4d", ifn->if_snd.ifq_maxlen);
    printf(" %-4d", ifn->if_snd.ifq_drops);
  }
  putchar('\n');
}


main(argc,argv)
  int argc;
  char *argv[];
{
  struct ifnet ifn;
  long addr;
  int i;
  

  for (i = 1; i < argc && argv[i][0] == '-'; i++)
    switch (argv[i][1])
    {
      case 'v':
        verbose_flag = 1;
	break;

      case 'V':
	puts(VERSION);
	break;

      default:
	puts("usage: ifstat [-v] [-V]");
	exit(1);
    }
  
  if ((fp = fopen(_PATH_KMEM, "r")) == NULL)
  {
    fprintf(stderr, "ifstat: ");
    perror(_PATH_KMEM);
    exit(1);
  }

  if (nlist(_PATH_UNIX, nl) != 0)
  {
    perror("ifstat: nlist");
    exit(1);
  }

  getbuf(nl[N_IFNET].n_value, &addr, sizeof(addr)); 

  printf("Name  Mtu   Net/Dest        %sAddress         Ipkts   Ierrs Opkts   Oerrs Collis%s\n",
	 verbose_flag ? "Broadcast       " : "",
	 verbose_flag ? " Tout Qlen Qmax Qdrp" : "");

  while (addr)
  {
    getbuf(addr, &ifn, sizeof(ifn));

    showinfo(&ifn);
    
    addr = (long) ifn.if_next;
  }
}
